@@ -1,5 +1,5 @@
 #!/bin/sh
-# Copyright 2005-2017 ECMWF.
+# Copyright 2005-2018 ECMWF.
 #
 # This software is licensed under the terms of the Apache Licence Version 2.0
 # which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -64,4 +64,12 @@ set -e
 
 diff ${MYLOG} ${REF}
 
+# Test for operator 203YYY
+# --------------------------
+INPUT=israel_observations_2017041010.bufr
+rm -f ${TEMP}
+$PYTHON ${examples_src}/bufr_copy_data2.py ${INPUT} ${TEMP}
+${tools_dir}/bufr_compare -b meanWindDirectionForSurfaceTo1500M,unexpandedDescriptors ${TEMP} ${INPUT}
+
+
 rm -f ${TEMP} ${REF} ${MYLOG}
