@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2017 ECMWF.
+ * Copyright 2005-2018 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -144,10 +144,18 @@ static int destroy(grib_dumper* d)
     return GRIB_SUCCESS;
 }
 
+static char* lval_to_string(grib_context* c, long v)
+{
+    char* sval=grib_context_malloc_clear(c,sizeof(char)*40);
+    if (v == GRIB_MISSING_LONG) sprintf(sval,"CODES_MISSING_LONG");
+    else                        sprintf(sval,"%ld",v);
+    return sval;
+}
 static char* dval_to_string(const grib_context* c,double v)
 {
     char* sval=(char*)grib_context_malloc_clear(c,sizeof(char)*40);
-    sprintf(sval,"%.18e",v);
+    if (v == GRIB_MISSING_DOUBLE) sprintf(sval,"CODES_MISSING_DOUBLE");
+    else                          sprintf(sval,"%.18e",v);
     return sval;
 }
 
@@ -196,7 +204,7 @@ static void dump_values(grib_dumper* d,grib_accessor* a)
         grib_context_free(c,sval);
 
         depth-=2;
-        /* Note: In python to make a tuple with one element, you need the trailing comma */
+        /* Note: In Python to make a tuple with one element, you need the trailing comma */
         fprintf(self->dumper.out,",)\n");
         grib_context_free(c,values);
 
@@ -206,16 +214,12 @@ static void dump_values(grib_dumper* d,grib_accessor* a)
             fprintf(self->dumper.out,"    codes_set_array(ibufr, '%s', rvalues)\n",a->name);
     } else {
         r=compute_bufr_key_rank(h,self->keys,a->name);
-        if( !grib_is_missing_double(a,value) ) {
-
-            sval=dval_to_string(c,value);
-            if (r!=0)
-                fprintf(self->dumper.out,"    codes_set(ibufr, '#%d#%s', %s)\n",r,a->name,sval);
-            else
-                fprintf(self->dumper.out,"    codes_set(ibufr, '%s', %s)\n",a->name,sval);
-
-            grib_context_free(c,sval);
-        }
+        sval=dval_to_string(c,value);
+        if (r!=0)
+            fprintf(self->dumper.out,"    codes_set(ibufr, '#%d#%s', %s)\n",r,a->name,sval);
+        else
+            fprintf(self->dumper.out,"    codes_set(ibufr, '%s', %s)\n",a->name,sval);
+        grib_context_free(c,sval);
     }
 
     if (self->isLeaf==0) {
@@ -287,14 +291,9 @@ static void dump_values_attribute(grib_dumper* d,grib_accessor* a, const char* p
 
         fprintf(self->dumper.out,"    codes_set_array(ibufr, '%s->%s' \n, rvalues)\n",prefix,a->name);
     } else {
-        /* int r=compute_bufr_key_rank(h,self->keys,a->name); */
-        if( !grib_is_missing_double(a,value) ) {
-
-            sval=dval_to_string(c,value);
-            fprintf(self->dumper.out,"    codes_set(ibufr, '%s->%s' \n,%s)\n",prefix,a->name,sval);
-
-            grib_context_free(c,sval);
-        }
+        sval=dval_to_string(c,value);
+        fprintf(self->dumper.out,"    codes_set(ibufr, '%s->%s' \n,%s)\n",prefix,a->name,sval);
+        grib_context_free(c,sval);
     }
 
     if (self->isLeaf==0) {
@@ -321,6 +320,7 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
     int i,r,icount;
     int cols=4;
     long count=0;
+    char* sval = NULL;
     grib_context* c=a->context;
     grib_handle* h=grib_handle_of_accessor(a);
 
@@ -379,23 +379,23 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
             fprintf(self->dumper.out,"    codes_set_array(ibufr, '%s', ivalues)\n",a->name);
 
     } else {
+        int doing_unexpandedDescriptors=0;
         r=compute_bufr_key_rank(h,self->keys,a->name);
-        if( !grib_is_missing_long(a,value) ) {
-            int doing_unexpandedDescriptors=0;
-            if (r!=0) {
-                fprintf(self->dumper.out,"    codes_set(ibufr, '#%d#%s', ",r,a->name);
-            } else {
-                if (strcmp(a->name, "unexpandedDescriptors")==0) {
-                    doing_unexpandedDescriptors=1;
-                    fprintf(self->dumper.out,"\n    # Create the structure of the data section\n");
-                }
-                fprintf(self->dumper.out,"    codes_set(ibufr, '%s', ",a->name);
+        sval=lval_to_string(c,value);
+        if (r!=0) {
+            fprintf(self->dumper.out,"    codes_set(ibufr, '#%d#%s', ",r,a->name);
+        } else {
+            if (strcmp(a->name, "unexpandedDescriptors")==0) {
+                doing_unexpandedDescriptors=1;
+                fprintf(self->dumper.out,"\n    # Create the structure of the data section\n");
             }
-
-            fprintf(self->dumper.out,"%ld)\n",value);
-            if (doing_unexpandedDescriptors)
-                fprintf(self->dumper.out,"\n");
+            fprintf(self->dumper.out,"    codes_set(ibufr, '%s', ",a->name);
         }
+
+        fprintf(self->dumper.out,"%s)\n",sval);
+        grib_context_free(c,sval);
+        if (doing_unexpandedDescriptors)
+            fprintf(self->dumper.out,"\n");
     }
 
     if (self->isLeaf==0) {
@@ -460,11 +460,10 @@ static void dump_long_attribute(grib_dumper* d, grib_accessor* a, const char* pr
         fprintf(self->dumper.out,"    codes_set_array(ibufr, '%s->%s' \n,ivalues)\n",prefix,a->name);
 
     } else {
-        /* int r=compute_bufr_key_rank(h,self->keys,a->name); */
-        if( !grib_is_missing_long(a,value) ) {
-            fprintf(self->dumper.out,"    codes_set(ibufr, '%s->%s'\n,",prefix,a->name);
-            fprintf(self->dumper.out,"%ld)\n",value);
-        }
+        char* sval=lval_to_string(c,value);
+        fprintf(self->dumper.out,"    codes_set(ibufr, '%s->%s'\n,",prefix,a->name);
+        fprintf(self->dumper.out,"%s)\n",sval);
+        grib_context_free(c,sval);
     }
 
     if (self->isLeaf==0) {
@@ -501,15 +500,13 @@ static void dump_double(grib_dumper* d,grib_accessor* a,const char* comment)
     self->empty=0;
 
     r=compute_bufr_key_rank(h,self->keys,a->name);
-    if( !grib_is_missing_double(a,value) ) {
-        sval=dval_to_string(c,value);
-        if (r!=0)
-            fprintf(self->dumper.out,"    codes_set(ibufr, '#%d#%s', %s)\n",r,a->name,sval);
-        else
-            fprintf(self->dumper.out,"    codes_set(ibufr, '%s', %s)\n",a->name,sval);
 
-        grib_context_free(c,sval);
-    }
+    sval=dval_to_string(c,value);
+    if (r!=0)
+        fprintf(self->dumper.out,"    codes_set(ibufr, '#%d#%s', %s)\n",r,a->name,sval);
+    else
+        fprintf(self->dumper.out,"    codes_set(ibufr, '%s', %s)\n",a->name,sval);
+    grib_context_free(c,sval);
 
     if (self->isLeaf==0) {
         char* prefix;
@@ -621,8 +618,9 @@ static void dump_string(grib_dumper* d,grib_accessor* a,const char* comment)
     err = grib_unpack_string(a,value,&size);
     p=value;
     r=compute_bufr_key_rank(h,self->keys,a->name);
-    if (grib_is_missing_string(a,(unsigned char *)value,size))
-        return;
+    if (grib_is_missing_string(a,(unsigned char *)value,size)) {
+        strcpy(value, ""); /* Empty string means MISSING string */
+    }
 
     while(*p) { if(!isprint(*p)) *p = '.'; p++; }
 
@@ -670,6 +668,7 @@ static void _dump_long_array(grib_handle* h, FILE* f, const char* key, const cha
     int cols=9,icount=0;
 
     if (grib_get_size(h,key,&size)==GRIB_NOT_FOUND) return;
+    if (size==0) return;
 
     fprintf(f,"    ivalues = (");
 
@@ -703,6 +702,7 @@ static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accesso
         _dump_long_array(h,self->dumper.out,"delayedDescriptorReplicationFactor","inputDelayedDescriptorReplicationFactor");
         _dump_long_array(h,self->dumper.out,"shortDelayedDescriptorReplicationFactor","inputShortDelayedDescriptorReplicationFactor");
         _dump_long_array(h,self->dumper.out,"extendedDelayedDescriptorReplicationFactor","inputExtendedDelayedDescriptorReplicationFactor");
+        _dump_long_array(h,self->dumper.out,"inputOverriddenReferenceValues","inputOverriddenReferenceValues");
         grib_dump_accessors_block(d,block);
         depth-=2;
     } else if (!grib_inline_strcmp(a->name,"groupNumber")) {
