@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2017 ECMWF.
+ * Copyright 2005-2018 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -43,7 +43,7 @@ static void init()
     pthread_mutex_init(&mutex_file,&attr);
     pthread_mutex_init(&mutex_rules,&attr);
     pthread_mutex_init(&mutex_concept,&attr);
-	pthread_mutex_init(&mutex_hash_array,&attr);
+    pthread_mutex_init(&mutex_hash_array,&attr);
     pthread_mutex_init(&mutex_stream,&attr);
     pthread_mutex_init(&mutex_parse,&attr);
     pthread_mutexattr_destroy(&attr);
@@ -174,7 +174,8 @@ int grib_recompose_name(grib_handle* h, grib_accessor *observer, const char* una
     return GRIB_SUCCESS;
 }
 
-int grib_accessor_print(grib_accessor* a,const char* name,int type,const char* format,const char* separator,int maxcols,int* newline,FILE* out)
+int grib_accessor_print(grib_accessor* a,const char* name,int type,const char* format,
+                        const char* separator,int maxcols,int* newline,FILE* out)
 {
     size_t size=0;
     char* sval=NULL;
@@ -285,7 +286,8 @@ int grib_accessor_print(grib_accessor* a,const char* name,int type,const char* f
     return ret;
 }
 
-int grib_accessors_list_print(grib_handle* h,grib_accessors_list* al,const char* name,int type,const char* format,const char* separator,int maxcols,int* newline,FILE* out)
+int grib_accessors_list_print(grib_handle* h, grib_accessors_list* al, const char* name,
+                              int type,const char* format,const char* separator,int maxcols,int* newline,FILE* out)
 {
     size_t size=0,len=0,replen=0;
     char* sval=NULL;
@@ -301,6 +303,9 @@ int grib_accessors_list_print(grib_handle* h,grib_accessors_list* al,const char*
     char default_separator[]=" ";
     grib_accessor* a=al->accessor;
 
+    /* Number of columns specified as 0 means print on ONE line i.e. num cols = infinity */
+    if (maxcols == 0) maxcols = INT_MAX;
+
     if (type==-1) type=grib_accessor_get_native_type(al->accessor);
     grib_accessors_list_value_count(al,&size);
     switch (type) {
@@ -310,7 +315,11 @@ int grib_accessors_list_print(grib_handle* h,grib_accessors_list* al,const char*
             char sbuf[1024]={0,};
             len = sizeof(sbuf);
             ret = grib_unpack_string(al->accessor,sbuf,&len);
-            fprintf(out,"%s",sbuf);
+            if (grib_is_missing_string(al->accessor,(unsigned char *)sbuf,len)) {
+                fprintf(out,"%s","MISSING");
+            } else {
+                fprintf(out,"%s",sbuf);
+            }
         } else {
             int i=0;
             int cols=0;
@@ -386,7 +395,8 @@ int grib_accessors_list_print(grib_handle* h,grib_accessors_list* al,const char*
         *newline=0;
         break;
     default:
-        grib_context_log(h->context, GRIB_LOG_WARNING,"grib_accessor_print: Problem to print \"%s\", invalid type %d", a->name,type);
+        grib_context_log(h->context, GRIB_LOG_WARNING,
+                         "grib_accessor_print: Problem printing \"%s\", invalid type %d", a->name, grib_get_type_name(type));
     }
     return ret;
 }
@@ -407,6 +417,7 @@ int grib_recompose_print(grib_handle* h, grib_accessor *observer, const char* un
     char buff1[1024]={0,};
     int maxcolsd=8;
     int maxcols;
+    long numcols = 0;
     int newline=1;
     const size_t uname_len = strlen(uname);
 
@@ -437,8 +448,13 @@ int grib_recompose_print(grib_handle* h, grib_accessor *observer, const char* un
                 break;
             case '!':
                 pp=(char*)uname;
-                maxcols=strtol(uname+i+1,&pp,10);
-                if (maxcols==0) maxcols=maxcolsd;
+                if (string_to_long(uname+i+1, &numcols)==GRIB_SUCCESS) {
+                    maxcols=(int)numcols;
+                } else {
+                    /* Columns specification is invalid integer */
+                    maxcols=maxcolsd;
+                }
+                strtol(uname+i+1,&pp,10);
                 while(pp && *pp!='%' && *pp!='!' && *pp!=']' && *pp!=':' && *pp!='\'' ) pp++;
                 i+=pp-uname-i-1;
                 break;
@@ -459,9 +475,7 @@ int grib_recompose_print(grib_handle* h, grib_accessor *observer, const char* un
 
                     if(ret != GRIB_SUCCESS)
                     {
-                        /*
-                 grib_context_log(h->context, GRIB_LOG_ERROR,"grib_recompose_print: Could not recompose print : %s", uname);
-                         */
+                        /* grib_context_log(h->context, GRIB_LOG_ERROR,"grib_recompose_print: Could not recompose print : %s", uname); */
                         return ret;
                     }
                 }
@@ -477,7 +491,6 @@ int grib_recompose_print(grib_handle* h, grib_accessor *observer, const char* un
             fprintf(out,"%c",uname[i]);
             type=-1;
         }
-
     }
     if (newline) fprintf(out,"\n");
 
