@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2016 ECMWF.
+ * Copyright 2005-2018 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -9,6 +9,7 @@
  */
 
 #include "grib_api_internal.h"
+#include <float.h>
 
 #ifdef ECCODES_ON_WINDOWS
  /* Replace C99/Unix rint() for Windows Visual C++ (only before VC++ 2013 versions) */
@@ -31,6 +32,8 @@
  #endif
 #endif
 
+typedef enum {eROUND_ANGLE_UP, eROUND_ANGLE_DOWN} RoundingPolicy;
+
 static void set_total_length(unsigned char* buffer,long *section_length,long *section_offset,int edition,size_t totalLength)
 {
     long off;
@@ -334,49 +337,78 @@ static grib_trie* init_list(const char* name)
     return 0;
 }
 
+/* For debugging purposes */
 static void print_values(grib_context* c, const grib_util_grid_spec2* spec,
-        const double* data_values, const size_t data_values_count, const grib_values *values, const int count)
+        const double* data_values, const size_t data_values_count,  /* the data pay load */
+        const grib_values *values, const size_t count)  /* keys and their values */
 {
-    int i;
-    printf("ECCODES DEBUG grib_util grib_set_values: setting %d values \n",count);
+    size_t i=0;
+    int isConstant = 1;
+    double v = 0, minVal=DBL_MAX, maxVal=-DBL_MAX;
+    printf("ECCODES DEBUG grib_util grib_set_values: setting %lu key/value pairs\n",(unsigned long)count);
 
-    for(i = 0; i < count ; i++)
+    for(i=0; i<count; i++)
     {
         switch(values[i].type)
         {
-        case GRIB_TYPE_LONG: printf("ECCODES DEBUG grib_util: => %s =  %ld;\n"
+        case GRIB_TYPE_LONG: printf("ECCODES DEBUG  grib_util: => %s =  %ld;\n"
                 ,values[i].name,(long)values[i].long_value); break;
-        case GRIB_TYPE_DOUBLE: printf("ECCODES DEBUG grib_util: => %s = %.16e;\n"
+        case GRIB_TYPE_DOUBLE: printf("ECCODES DEBUG  grib_util: => %s = %.16e;\n"
                 ,values[i].name,values[i].double_value); break;
-        case GRIB_TYPE_STRING: printf("ECCODES DEBUG grib_util: => %s = \"%s\";\n"
+        case GRIB_TYPE_STRING: printf("ECCODES DEBUG  grib_util: => %s = \"%s\";\n"
                 ,values[i].name,values[i].string_value); break;
         }
     }
 
-    if(spec->bitmapPresent) {
-        int missing = 0;
-        size_t j = 0;
-        double min = 1e100;
-        for(j = 0; j < data_values_count ; j++)
-        {
-            double d = data_values[j] - spec->missingValue;
-            if(d < 0) d = -d;
-
-            if(d < min) {
-                min = d;
+    printf("ECCODES DEBUG  grib_util: data_values_count=%lu;\n", (unsigned long)data_values_count);
+    for (i=0; i<data_values_count; i++) {
+        if (i==0) v = data_values[i];
+        if (data_values[i] != spec->missingValue) {
+            if (v == spec->missingValue) {
+                v = data_values[i];
+            } else if (v != data_values[i]) {
+                isConstant=0;
+                break;
             }
-
-            if(data_values[j] == spec->missingValue)
-                missing++;
         }
     }
+    
+    for (i=0; i<data_values_count; i++) {
+        v = data_values[i];
+        if (v!=spec->missingValue) {
+            if (v < minVal) minVal=v;
+            if (v > maxVal) maxVal=v;
+        }
+    }
+    printf("ECCODES DEBUG  grib_util: data_values are CONSTANT? %d\t(min=%.16e, max=%.16e)\n",
+           isConstant, minVal, maxVal);
+
+#if 0
+        if (spec->bitmapPresent) {
+            int missing = 0;
+            size_t j = 0;
+            double min = 1e100;
+            for(j = 0; j < data_values_count ; j++)
+            {
+                double d = data_values[j] - spec->missingValue;
+                if(d < 0) d = -d;
+                if(d < min) {
+                    min = d;
+                }
+                if(data_values[j] == spec->missingValue)
+                    missing++;
+            }
+        }
+#endif
 }
 
+/*
 static int DBL_EQUAL(double d1, double d2, double tolerance)
 {
     return fabs(d1-d2) < tolerance;
 }
-
+*/
+#if 0
 /* Returns a boolean: 1 if angle can be encoded, 0 otherwise */
 static int grib1_angle_can_be_encoded(const double angle)
 {
@@ -389,8 +421,119 @@ static int grib1_angle_can_be_encoded(const double angle)
     return 0; /* sub millidegree. Cannot be encoded in grib1 */
 }
 
+/* Returns a boolean: 1 if angle can be encoded, 0 otherwise */
+static int angle_can_be_encoded(const double angle, const double angular_precision)
+{
+    const double angle_expanded = angle * angular_precision;
+    Assert(angular_precision>0);
+    double rounded = (long)(angle_expanded+0.5)/angular_precision;
+    if (angle<0) {
+        rounded = (long)(angle_expanded-0.5)/angular_precision;
+    }
+    if (angle == rounded) return 1;
+    /*printf("      ......... angle cannot be encoded: %.10e\n", angle);*/
+    return 0; /* Cannot be encoded */
+}
+#endif
+
+/* Returns a boolean: 1 if angle can be encoded, 0 otherwise */
+static int angle_can_be_encoded(grib_handle* h, const double angle)
+{
+    int ret = 0;
+    int retval = 1;
+    grib_handle* h2 = NULL;
+    char sample_name[16] = {0,};
+    long angular_precision = 0; /* e.g. 1e3 for grib1 and 1e6 for grib2 */
+    long edition = 0, coded = 0;
+    double expanded, diff;
+
+    if((ret = grib_get_long(h,"edition",&edition)) != 0) return ret;
+    if((ret = grib_get_long(h, "angularPrecision", &angular_precision)) != 0) return ret;
+    Assert(angular_precision > 0);
+
+    sprintf(sample_name, "GRIB%ld", edition);
+    h2 = grib_handle_new_from_samples(0, sample_name);
+    if((ret = grib_set_double(h2, "latitudeOfFirstGridPointInDegrees", angle)) != 0) return ret;
+    if((ret = grib_get_long(h2,   "latitudeOfFirstGridPoint", &coded)) != 0) return ret;
+    grib_handle_delete(h2);
+
+    expanded = angle*angular_precision;
+    diff = fabs(expanded - coded);
+    if (diff < 1.0/angular_precision)
+        retval = 1;
+    else
+        retval = 0;
+
+    return retval;
+}
+
+#ifdef ECCODES_ON_WINDOWS
+#define round(a) ( (a) >=0 ? ((a)+0.5) : ((a)-0.5) )
+#endif
+static double adjust_angle(const double angle, const RoundingPolicy policy, const double angular_precision)
+{
+    double result = 0;
+    Assert(angular_precision > 0);
+    result = angle * angular_precision;
+    if (policy == eROUND_ANGLE_UP)
+        result = round(result+0.5);
+    else
+        result = round(result-0.5);
+    result = result / angular_precision;
+    return result;
+}
+
+/* Search key=value array for:
+ *   latitudeOfFirstGridPointInDegrees
+ *   longitudeOfFirstGridPointInDegrees
+ *   latitudeOfLastGridPointInDegrees
+ *   longitudeOfLastGridPointInDegrees
+ * and change their values to expand the bounding box
+ */
+static int expand_bounding_box(grib_handle* h, grib_values *values, const size_t count)
+{
+    int ret = GRIB_SUCCESS;
+    size_t i=0;
+    double new_angle = 0;
+    RoundingPolicy roundingPolicy = eROUND_ANGLE_UP;
+    long angular_precision = 0; /* e.g. 1e3 for grib1 and 1e6 for grib2 */
+    if((ret = grib_get_long(h, "angularPrecision", &angular_precision)) != 0)
+        return ret;
+
+    for(i=0; i<count; i++) {
+        int is_angle = 0;
+        if (strcmp(values[i].name, "longitudeOfFirstGridPointInDegrees")==0) {
+            roundingPolicy = eROUND_ANGLE_DOWN;
+            is_angle = 1;
+        }
+        else if (strcmp(values[i].name, "longitudeOfLastGridPointInDegrees")==0) {
+            roundingPolicy = eROUND_ANGLE_UP;
+            is_angle = 1;
+        }
+        else if (strcmp(values[i].name, "latitudeOfFirstGridPointInDegrees")==0) {
+            roundingPolicy = eROUND_ANGLE_UP;
+            is_angle = 1;
+        }
+        else if (strcmp(values[i].name, "latitudeOfLastGridPointInDegrees")==0) {
+            roundingPolicy = eROUND_ANGLE_DOWN;
+            is_angle = 1;
+        }
+
+        if (is_angle && !angle_can_be_encoded(h, values[i].double_value)) {
+            new_angle = adjust_angle(values[i].double_value, roundingPolicy, angular_precision);
+            if (h->context->debug) {
+                printf("ECCODES DEBUG  grib_util EXPAND_BOUNDING_BOX %s: old=%.15e new=%.15e (%s)\n",
+                       values[i].name, values[i].double_value, new_angle,
+                       (roundingPolicy==eROUND_ANGLE_UP?"Up":"Down"));
+            }
+            values[i].double_value = new_angle;
+        }
+    }
+    return ret;
+}
+
 /* Returns a boolean: 1 if angle is too small, 0 otherwise */
-static int angle_too_small(const double angle, const double angular_precision)
+/*static int angle_too_small(const double angle, const double angular_precision)
 {
     const double a = fabs(angle);
     if (a > 0 && a < angular_precision) return 1;
@@ -402,8 +545,9 @@ static double normalise_angle(double angle)
     while (angle<0)   angle += 360;
     while (angle>360) angle -= 360;
     return angle;
-}
+}*/
 
+#if 0
 /* Check what is coded in the handle is what is requested by the spec. */
 /* Return GRIB_SUCCESS if the geometry matches, otherwise the error code */
 static int check_handle_against_spec(grib_handle* handle, const long edition,
@@ -529,6 +673,7 @@ static int check_handle_against_spec(grib_handle* handle, const long edition,
     }
     return GRIB_SUCCESS;
 }
+#endif
 
 static const char* get_grid_type_name(const int spec_grid_type)
 {
@@ -666,9 +811,11 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     double laplacianOperator;
     int packingTypeIsSet=0;
     int setSecondOrder=0;
+    int setJpegPacking=0;
     size_t slen=17;
     int grib1_high_resolution_fix = 0; /* boolean: See GRIB-863 */
     int global_grid = 0; /* boolean */
+    int expandBoundingBox = 0;
 
     static grib_util_packing_spec default_packing_spec = {0, };
     Assert(h);
@@ -687,11 +834,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     }
 
     if (packing_spec->deleteLocalDefinition) {
-        /* TODO: We need two calls because of grib1/grib2 issues re removing local defs! */
-        if (editionNumber==1){
-            SET_LONG_VALUE("deleteLocalDefinition",1);
-        }
-        SET_LONG_VALUE("setLocalDefinition", 0);
+        SET_LONG_VALUE("deleteLocalDefinition",1);
     }
 
     len=100;
@@ -738,7 +881,8 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                 break;
             default :
                 printf("invalid packing_spec->packing_type = %ld\n",(long)packing_spec->packing_type);
-                abort();
+                *err = GRIB_INTERNAL_ERROR;
+                goto cleanup;
                 break;
             }
             packingTypeIsSet=1;
@@ -763,7 +907,8 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
 
         default:
             printf("invalid packing_spec->accuracy = %ld\n",(long)packing_spec->accuracy);
-            abort();
+            *err = GRIB_INTERNAL_ERROR;
+            goto cleanup;
             break;
         }
 
@@ -773,8 +918,9 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
             return h;
         }
 
-        if (h->context->debug==-1)
+        if (h->context->debug==-1) {
             print_values(h->context,spec,data_values,data_values_count,values,count);
+        }
 
         if((*err = grib_set_values(h,values,count)) != 0)
         {
@@ -899,8 +1045,8 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         SET_LONG_VALUE  ("ijDirectionIncrementGiven",    1);
         if (editionNumber == 1) {
             /* GRIB-863: GRIB1 cannot represent increments less than a millidegree */
-            if (!grib1_angle_can_be_encoded(spec->iDirectionIncrementInDegrees) ||
-                    !grib1_angle_can_be_encoded(spec->jDirectionIncrementInDegrees))
+            if (!angle_can_be_encoded(h, spec->iDirectionIncrementInDegrees) ||
+                !angle_can_be_encoded(h, spec->jDirectionIncrementInDegrees))
             {
                 grib1_high_resolution_fix = 1;
                 /* Set flag to compute the increments */
@@ -1071,8 +1217,12 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                 SET_STRING_VALUE("packingType","grid_complex");
             break;
         case GRIB_UTIL_PACKING_TYPE_JPEG:
+            /* Have to delay JPEG packing:
+             * Reason 1: It is not available in GRIB1 and so we have to wait until we change edition
+             * Reason 2: It has to be done AFTER we set the data values
+             */
             if (strcmp(input_packing_type,"grid_jpeg") && !strcmp(input_packing_type,"grid_simple"))
-                SET_STRING_VALUE("packingType","grid_jpeg");
+                setJpegPacking = 1;
             break;
         case GRIB_UTIL_PACKING_TYPE_GRID_SECOND_ORDER:
             /* we delay the set of grid_second_order because we don't want
@@ -1081,7 +1231,8 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
             break;
         default :
             printf("invalid packing_spec->packing_type = %ld\n",(long)packing_spec->packing_type);
-            abort();
+            *err = GRIB_INTERNAL_ERROR;
+            goto cleanup;
             break;
         }
     }
@@ -1133,20 +1284,32 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
 
     default:
         printf("invalid packing_spec->accuracy = %ld\n",(long)packing_spec->accuracy);
-        abort();
+        *err = GRIB_INTERNAL_ERROR;
+        goto cleanup;
         break;
     }
 
     if(packing_spec->extra_settings_count) {
         for(i = 0; i < packing_spec->extra_settings_count; i++) {
             Assert(count < 1024);
-            values[count++] = packing_spec->extra_settings[i];
-            if (strcmp(packing_spec->extra_settings[i].name, "global")==0 &&
-                    packing_spec->extra_settings[i].long_value == 1)
+            if (strcmp(packing_spec->extra_settings[i].name, "expandBoundingBox")==0)
             {
-                /* GRIB-922: Request is for a global grid. Setting this key will
-                 * calculate the lat/lon values. So the spec's lat/lon can be ignored */
-                global_grid = 1;
+                if (packing_spec->extra_settings[i].long_value == 1) {
+                    /* ECC-625: Request is for expansion of bounding box (sub-area).
+                     * This is also called the "snap-out" policy */
+                    expandBoundingBox = 1;
+                }
+            }
+            else
+            {
+                values[count++] = packing_spec->extra_settings[i];
+                if (strcmp(packing_spec->extra_settings[i].name, "global")==0 &&
+                        packing_spec->extra_settings[i].long_value == 1)
+                {
+                    /* GRIB-922: Request is for a global grid. Setting this key will
+                     * calculate the lat/lon values. So the spec's lat/lon can be ignored */
+                    global_grid = 1;
+                }
             }
         }
     }
@@ -1182,8 +1345,20 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         }
     }
 
-    if (h->context->debug==-1)
+    if (h->context->debug==-1) {
+        printf("ECCODES DEBUG grib_util: global_grid = %d\n", global_grid);
+        printf("ECCODES DEBUG grib_util: expandBoundingBox = %d\n", expandBoundingBox);
         print_values(h->context,spec,data_values,data_values_count,values,count);
+    }
+
+    /* Apply adjustments to bounding box if needed */
+    if (expandBoundingBox) {
+        if ((*err=expand_bounding_box(outh, values, count)) != 0)
+        {
+            fprintf(stderr,"SET_GRID_DATA_DESCRIPTION: Cannot expand bounding box: %s\n",grib_get_error_message(*err));
+            goto cleanup;
+        }
+    }
 
     if((*err = grib_set_values(outh,values,count)) != 0)
     {
@@ -1198,15 +1373,15 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     if((*err = grib_set_double_array(outh,"values",data_values,data_values_count)) != 0)
     {
         FILE* ferror;
-        long i,lcount;
+        size_t ii,lcount;
         grib_context* c=grib_context_get_default();
 
         ferror=fopen("error.data","w");
         lcount=0;
         fprintf(ferror,"# data_values_count=%ld\n",(long)data_values_count);
         fprintf(ferror,"set values={ ");
-        for (i=0;i<data_values_count-1;i++) {
-            fprintf(ferror,"%g, ",data_values[i]);
+        for (ii=0;ii<data_values_count-1;ii++) {
+            fprintf(ferror,"%g, ",data_values[ii]);
             if (lcount>10) {fprintf(ferror,"\n");lcount=0;}
             lcount++;
         }
@@ -1294,14 +1469,28 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         }
     }
 
+    if (editionNumber > 1 || packing_spec->editionNumber > 1) {
+        /* ECC-353 */
+        /* JPEG packing is not available in GRIB edition 1 and has to be done AFTER we set data values */
+        if (setJpegPacking == 1) {
+            *err = grib_set_string(outh, "packingType", "grid_jpeg", &slen);
+            if (*err != GRIB_SUCCESS) {
+                fprintf(stderr,"GRIB_UTIL_SET_SPEC: Failed to change packingType to JPEG: %s\n",
+                        grib_get_error_message(*err));
+                goto cleanup;
+            }
+        }
+    }
+
     if (packing_spec->deleteLocalDefinition) {
-        /* TODO: We need two calls because of grib1/grib2 issues re removing local defs! */
-        grib_set_long(outh,"setLocalDefinition", 0);
         grib_set_long(outh,"deleteLocalDefinition", 1);
     }
 
+    /* Disable check: need to re-examine GRIB-864 */
+#if 0
     if ( (*err = check_handle_against_spec(outh, editionNumber, spec, global_grid)) != GRIB_SUCCESS)
     {
+
         grib_context* c=grib_context_get_default();
         fprintf(stderr,"GRIB_UTIL_SET_SPEC: Geometry check failed! %s\n", grib_get_error_message(*err));
         if (editionNumber == 1) {
@@ -1311,7 +1500,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
             grib_write_message(outh,"error.grib","w");
         goto cleanup;
     }
-
+#endif
     if (h->context->debug==-1)
         printf("ECCODES DEBUG: grib_util_set_spec end\n");
 
@@ -1401,6 +1590,7 @@ int grib_moments(grib_handle* h,double east,double north,double west,double sout
     grib_context_free(c,lat);
     grib_context_free(c,lon);
     grib_context_free(c,values);
+    (void)mass;
 
     return ret;
 }
@@ -1618,116 +1808,37 @@ char get_dir_separator_char(void)
     return DIR_SEPARATOR_CHAR;
 }
 
-/* Return the component after final slash */
-/*  "/tmp/x"  -> "x"  */
-/*  "/tmp/"   -> ""   */
-const char* extract_filename(const char* filepath)
-{
-    const char* s = strrchr(filepath, get_dir_separator_char());
-    if (!s) return filepath;
-    else    return s + 1;
-}
-
-/* Boolean return type: 1 if the reduced gaussian field is global, 0 for sub area */
-int is_gaussian_global(
-        double lat1, double lat2, double lon1, double lon2,/* bounding box*/
-        long num_points_equator, /* num points on latitude at equator */
-        const double* latitudes, /* array of Gaussian latitudes (size 2*N) */
-        double angular_precision /* tolerance for angle comparison */
-)
-{
-    int global = 1;
-    const double d = fabs(latitudes[0] - latitudes[1]);
-    /* Compute the expected last longitude for a global field */
-    const double lon2_global = 360.0 - 360.0/num_points_equator;
-    /* Compute difference between expected longitude and actual one */
-    const double lon2_diff = fabs( lon2  - lon2_global ) - 360.0/num_points_equator;
-
-    /* Note: final gaussian latitude = -first latitude */
-    if ( (fabs(lat1 - latitudes[0]) >= d ) ||
-         (fabs(lat2 + latitudes[0]) >= d ) ||
-         lon1 != 0                         ||
-         lon2_diff > angular_precision
-    )
-    {
-        global = 0; /* sub area */
-    }
-    return global;
-}
-
 char* codes_getenv(const char* name)
 {
     /* Look for the new ecCodes environment variable names */
     /* if not found, then look for old grib_api ones for backward compatibility */
-    const char* old_name = name;
-    char* result = NULL;
-
-    /* Test the most commonly used variables first */
-    if      (STR_EQ(name, "ECCODES_SAMPLES_PATH")) old_name="GRIB_SAMPLES_PATH";
-    else if (STR_EQ(name, "ECCODES_DEFINITION_PATH")) old_name="GRIB_DEFINITION_PATH";
-    else if (STR_EQ(name, "ECCODES_DEBUG")) old_name="GRIB_API_DEBUG";
-
-    else if (STR_EQ(name, "ECCODES_FAIL_IF_LOG_MESSAGE")) old_name="GRIB_API_FAIL_IF_LOG_MESSAGE";
-    else if (STR_EQ(name, "ECCODES_GRIB_WRITE_ON_FAIL")) old_name="GRIB_API_WRITE_ON_FAIL";
-    else if (STR_EQ(name, "ECCODES_GRIB_LARGE_CONSTANT_FIELDS")) old_name="GRIB_API_LARGE_CONSTANT_FIELDS";
-    else if (STR_EQ(name, "ECCODES_NO_ABORT")) old_name="GRIB_API_NO_ABORT";
-    else if (STR_EQ(name, "ECCODES_GRIBEX_MODE_ON")) old_name="GRIB_GRIBEX_MODE_ON";
-    else if (STR_EQ(name, "ECCODES_GRIB_IEEE_PACKING")) old_name="GRIB_IEEE_PACKING";
-    else if (STR_EQ(name, "ECCODES_IO_BUFFER_SIZE")) old_name="GRIB_API_IO_BUFFER_SIZE";
-    else if (STR_EQ(name, "ECCODES_LOG_STREAM")) old_name="GRIB_API_LOG_STREAM";
-    else if (STR_EQ(name, "ECCODES_GRIB_NO_BIG_GROUP_SPLIT")) old_name="GRIB_API_NO_BIG_GROUP_SPLIT";
-    else if (STR_EQ(name, "ECCODES_GRIB_NO_SPD")) old_name="GRIB_API_NO_SPD";
-    else if (STR_EQ(name, "ECCODES_GRIB_KEEP_MATRIX")) old_name="GRIB_API_KEEP_MATRIX";
-    else if (STR_EQ(name, "_ECCODES_ECMWF_TEST_DEFINITION_PATH")) old_name="_GRIB_API_ECMWF_TEST_DEFINITION_PATH";
-    else if (STR_EQ(name, "_ECCODES_ECMWF_TEST_SAMPLES_PATH")) old_name="_GRIB_API_ECMWF_TEST_SAMPLES_PATH";
-    else if (STR_EQ(name, "ECCODES_GRIB_JPEG")) old_name="GRIB_JPEG";
-    else if (STR_EQ(name, "ECCODES_GRIB_DUMP_JPG_FILE")) old_name="GRIB_DUMP_JPG_FILE";
-    else if (STR_EQ(name, "ECCODES_PRINT_MISSING")) old_name="GRIB_PRINT_MISSING";
-
-    result = getenv(name);
+    char* result = getenv(name);
     if (result == NULL) {
+        const char* old_name = name;
+
+        /* Test the most commonly used variables first */
+        if      (STR_EQ(name, "ECCODES_SAMPLES_PATH")) old_name="GRIB_SAMPLES_PATH";
+        else if (STR_EQ(name, "ECCODES_DEFINITION_PATH")) old_name="GRIB_DEFINITION_PATH";
+        else if (STR_EQ(name, "ECCODES_DEBUG")) old_name="GRIB_API_DEBUG";
+
+        else if (STR_EQ(name, "ECCODES_FAIL_IF_LOG_MESSAGE")) old_name="GRIB_API_FAIL_IF_LOG_MESSAGE";
+        else if (STR_EQ(name, "ECCODES_GRIB_WRITE_ON_FAIL")) old_name="GRIB_API_WRITE_ON_FAIL";
+        else if (STR_EQ(name, "ECCODES_GRIB_LARGE_CONSTANT_FIELDS")) old_name="GRIB_API_LARGE_CONSTANT_FIELDS";
+        else if (STR_EQ(name, "ECCODES_NO_ABORT")) old_name="GRIB_API_NO_ABORT";
+        else if (STR_EQ(name, "ECCODES_GRIBEX_MODE_ON")) old_name="GRIB_GRIBEX_MODE_ON";
+        else if (STR_EQ(name, "ECCODES_GRIB_IEEE_PACKING")) old_name="GRIB_IEEE_PACKING";
+        else if (STR_EQ(name, "ECCODES_IO_BUFFER_SIZE")) old_name="GRIB_API_IO_BUFFER_SIZE";
+        else if (STR_EQ(name, "ECCODES_LOG_STREAM")) old_name="GRIB_API_LOG_STREAM";
+        else if (STR_EQ(name, "ECCODES_GRIB_NO_BIG_GROUP_SPLIT")) old_name="GRIB_API_NO_BIG_GROUP_SPLIT";
+        else if (STR_EQ(name, "ECCODES_GRIB_NO_SPD")) old_name="GRIB_API_NO_SPD";
+        else if (STR_EQ(name, "ECCODES_GRIB_KEEP_MATRIX")) old_name="GRIB_API_KEEP_MATRIX";
+        else if (STR_EQ(name, "_ECCODES_ECMWF_TEST_DEFINITION_PATH")) old_name="_GRIB_API_ECMWF_TEST_DEFINITION_PATH";
+        else if (STR_EQ(name, "_ECCODES_ECMWF_TEST_SAMPLES_PATH")) old_name="_GRIB_API_ECMWF_TEST_SAMPLES_PATH";
+        else if (STR_EQ(name, "ECCODES_GRIB_JPEG")) old_name="GRIB_JPEG";
+        else if (STR_EQ(name, "ECCODES_GRIB_DUMP_JPG_FILE")) old_name="GRIB_DUMP_JPG_FILE";
+        else if (STR_EQ(name, "ECCODES_PRINT_MISSING")) old_name="GRIB_PRINT_MISSING";
+
         result = getenv(old_name);
     }
     return result;
 }
-
-/* Return the rank of the key using list of keys (For BUFR keys) */
-/* The argument 'keys' is an input as well as output from each call */
-int compute_key_rank(grib_handle* h, grib_string_list* keys, const char* key)
-{
-    grib_string_list* next=keys;
-    grib_string_list* prev=keys;
-    int theRank=0;
-    size_t size=0;
-    grib_context* c=h->context;
-    Assert(h->product_kind == PRODUCT_BUFR);
-
-    while (next && next->value && strcmp(next->value,key)) {
-        prev=next;
-        next=next->next;
-    }
-    if (!next) {
-        prev->next=(grib_string_list*)grib_context_malloc_clear(c,sizeof(grib_string_list));
-        next=prev->next;
-    }
-    if (!next->value) {
-        next->value=strdup(key);
-        next->count=0;
-    }
-
-    next->count++;
-    theRank=next->count;
-    if (theRank==1) {
-        /* If the count is 1 it could mean two things: */
-        /*   This is the first instance of the key and there is another one */
-        /*   This is the first and only instance of the key */
-        /* So we check if there is a second one of this key, */
-        /* If not, then rank is zero i.e. this is the only instance */
-        char* s=grib_context_malloc_clear(c,strlen(key)+5);
-        sprintf(s,"#2#%s",key);
-        if (grib_get_size(h,s,&size)==GRIB_NOT_FOUND) theRank=0;
-        grib_context_free(c, s);
-    }
-
-    return theRank;
-}
