@@ -1,5 +1,5 @@
 #!/bin/sh
-# Copyright 2005-2017 ECMWF.
+# Copyright 2005-2018 ECMWF.
 #
 # This software is licensed under the terms of the Apache Licence Version 2.0
 # which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -12,24 +12,11 @@
 
 REDIRECT=/dev/null
 
-# First check if Adaptive Entropy Coding (AEC) feature is enabled
-skip_test=0
+# Disable if autotools being used
 src_config=${src_dir}/config.h
 if [ -f ${src_config} ]; then
-  set +e
-  grep '#undef HAVE_LIBAEC' ${src_config} >/dev/null
-  status=$?
-  set -e
-  if [ $status -eq 0 ]; then
-    # Found the string so feature is disabled
-    skip_test=1
-  fi
-fi
-if [ $skip_test -eq 1 ]; then
-  #echo "AEC feature was not enabled. Skipping this test."
   exit 0
 fi
-echo "AEC feature was enabled."
 
 BLACKLIST="totalLength,section5Length,section7Length,dataRepresentationTemplateNumber"
 
