@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2017 ECMWF.
+ * Copyright 2005-2018 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -39,7 +39,7 @@ grib_option grib_options[]={
         {"i:",0,0,0,1,0}
 };
 
-char* grib_tool_description="Get values of some keys from a grib file."
+char* grib_tool_description="Get values of some keys from a GRIB file."
         "\n\tIt is similar to grib_ls, but fails returning an error code "
         "\n\twhen an error occurs (e.g. key not found).";
 char* grib_tool_name="grib_get";
