@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2017 ECMWF.
+ * Copyright 2005-2018 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -213,8 +213,8 @@ static int grib_tool_with_orderby(grib_runtime_options* options)
 
     set=grib_fieldset_new_from_files(0,filenames,files_count,0,0,0,options->orderby,&err);
     if (err) {
-        grib_context_log(c,GRIB_LOG_FATAL,"unable to create index for input file %s (%s)",
-                filenames[0],grib_get_error_message(err));
+        grib_context_log(c,GRIB_LOG_ERROR,"unable to create index for input file %s (%s)",
+                         filenames[0],grib_get_error_message(err));
         exit(err);
     }
 
@@ -286,7 +286,10 @@ static int grib_tool_without_orderby(grib_runtime_options* options)
     while (infile!=NULL && infile->name!=NULL) {
 
         if (options->print_statistics && options->verbose) fprintf(dump_file,"%s\n",infile->name);
-        infile->file = fopen(infile->name,"r");
+        if (strcmp(infile->name,"-")==0)
+            infile->file = stdin;
+        else
+            infile->file = fopen(infile->name,"r");
         if(!infile->file) {
             perror(infile->name);
             exit(1);
@@ -697,7 +700,7 @@ static void grib_tools_set_print_keys(grib_runtime_options* options, grib_handle
 
         grib_keys_iterator_delete(kiter);
         if (options->print_keys_count==0 && options->latlon == 0 ) {
-            int i=0,j=0,k=0,ns_count=0;
+            int j=0,k=0,ns_count=0;
             char* all_namespace_vals[1024] = {NULL,}; /* sorted array containing all namespaces */
             printf("ERROR: namespace \"%s\" does not contain any key.\n",ns);
             printf("Here are the available namespaces in this message:\n");
